#include "stdio.h"
#include "avr/io.h"
#include "stdlib.h"
#include "heatsystem.h"
#include "leash.c"
#include "symbols.c"
#include "menus.c"
#include "avr/interrupt.h"




void setup(void)
{

	SPI_init();

	ADMUX = 0b01000000; 	/* AVCC as ref, ADC channel 0 */
	ADCSRA = 0b10000111; 	/* ADC enabled, CLK/128 prescaler */
	ADCSRB = 0b00000000; 	/* Auto trigger is off */
	DIDR0 = 0b00111111;  	/* Disables digital input of ADC2 */
		
	TCCR1B = 0b00000101;	/* CLK/1024 */
	OCR1A = 7200;			/* 1sec OCIE1A interrupt */
	TIMSK1 = 0b00000010;	/* OCIE1A enabled */


	dout_ddr = 0xff;
	din_ddr = 0x00;
	PORTB = 0x0f;

	LCD_init();
	loadvars();
	sei();

	mcaltimer = vars[81]+vars[80];

}

unsigned int adcread(unsigned char chan)
{
	unsigned int result;
	/* selects channel, as well as voltage reference */
	ADMUX = (0b11000000 | chan); 
	/* Start ADC conversion */
	ADCSRA |= 0b01000000; 
	/* waits for conversion to be done */
	while(ADCSRA&0b01000000); 
	result = (ADC);
	ADCSRA |= 0b01000000; 
	/* waits for conversion to be done, do it twice because first conversion is garbage */
	while(ADCSRA&0b01000000); 
	result = (ADC);

	return result;
}

signed int gettempint(unsigned char chan)
{
	signed int t;

	t = adcread(chan);
	t = (((2.56*t)/1023)*100)-50;

	switch(chan){
    case solartemp:
        return t+vars[solartemp+90];
    case tanktoptemp:
        return t+vars[tanktoptemp+90];
    case tankmiddletemp:
        return t+vars[tankmiddletemp+90];
    case tankbottomtemp:
        return t+vars[tankbottomtemp+90];
    case outsidetemp:
        return t+vars[outsidetemp+90];
    case insidetemp:
        return t+vars[insidetemp+90];
    case returntemp:
        return t+vars[returntemp+90];
    case forwardtemp:
        return t+vars[forwardtemp+90];
    default:
        return t;
	}

}

double gettemp(unsigned char chan)
{
	double t;

	t = adcread(chan);
	t = (((2.56*t)/1023)*100)-50;

	switch(chan){
    case solartemp:
        return t+vars[solartemp+90];
    case tanktoptemp:
        return t+vars[tanktoptemp+90];
    case tankmiddletemp:
        return t+vars[tankmiddletemp+90];
    case tankbottomtemp:
        return t+vars[tankbottomtemp+90];
    case outsidetemp:
        return t+vars[outsidetemp+90];
    case insidetemp:
        return t+vars[insidetemp+90];
    case returntemp:
        return t+vars[returntemp+90];
    case forwardtemp:
        return t+vars[forwardtemp+90];
    default:
        return t;
	}

}

#ifdef internal
unsigned char getshuntpos(void)
{
	unsigned int p = (shunttime*100)/vars[81];
	return p;
}
#endif


#ifdef external
unsigned char getshuntpos(void)
{
	unsigned int p = adcread(shunt);
	p *= 100;
	p /= 1024;
	return p;
}
#endif


#ifdef internal

double getforwardsp(void)
{
double ot = 0;
double calc = 0;
	
	 ot = gettempint(outsidetemp);
	if((-25 < ot) && (ot < -20)){
		calc = vars[79]-vars[78];
		calc *= 100;
		calc /= 15;
		calc *= (-20-ot);
		calc /= 100;
		calc += vars[78];

	}

	if((-20 < ot) && (ot < -15)){
		calc = vars[78]-vars[77];
		calc *= 100;
		calc /= 15;
		calc *= (-15-ot);
		calc /= 100;
		calc += vars[77];

	}

	if((-15 < ot) && (ot < -10)){
		calc = vars[77]-vars[76];
		calc *= 100;
		calc /= 15;
		calc *= (-10-ot);
		calc /= 100;
		calc += vars[76];

	}

	if((-10 < ot) && (ot < -5)){
		calc = vars[76]-vars[75];
		calc *= 100;
		calc /= 15;
		calc *= (-5-ot);
		calc /= 100;
		calc += vars[75];

	}

	if((-5 < ot) && (ot < 0)){
		calc = vars[75]-vars[74];
		calc *= 100;
		calc /= 15;
		calc *= (ot);
		calc /= 100;
		calc += vars[74];

	}

	if((0 < ot) && (ot < 5)){
		calc = vars[74]-vars[73];
		calc *= 100;
		calc /= 15;
		calc *= (5-ot);
		calc /= 100;
		calc += vars[73];

	}

	if((5 < ot) && (ot < 10)){
		calc = vars[73]-vars[72];
		calc *= 100;
		calc /= 15;
		calc *= (10-ot);
		calc /= 100;
		calc += vars[72];

	}

	if((10 < ot) && (ot < 15)){
		calc = vars[72]-vars[71];
		calc *= 1000;
		calc /= 15;
		calc *= (15-ot);
		calc /= 1000;
		calc += vars[71];

	}

	if((15 < ot) && (ot < 20)){
		calc = vars[71]-vars[70];
		calc *= 1000;
		calc /= 15;
		calc *= (20-ot);
		calc /= 1000;
		calc += vars[70];

	
		FB_writestring(0,0,"Match");
		calc = -25-gettempint(outsidetemp);
		//calc *= 2;
		calc += vars[79];
		return calc;
	}
	
	
	

	
	
	

	if(gettempint(outsidetemp) <= -25)	return vars[79]+(-25-gettempint(outsidetemp));	
	if(gettempint(outsidetemp) == -25) return vars[79];
	if(gettempint(outsidetemp) == -20) return vars[78];
	if(gettempint(outsidetemp) == -15) return vars[77];
	if(gettempint(outsidetemp) == -10) return vars[76];
	if(gettempint(outsidetemp) == -5) return vars[75];
	if(gettempint(outsidetemp) == 0) return vars[74];
	if(gettempint(outsidetemp) == 5) return vars[73];
	if(gettempint(outsidetemp) == 10) return vars[72];
	if(gettempint(outsidetemp) == 15) return vars[71];
	if(gettempint(outsidetemp) == 20) return vars[70];

	return calc;

}

#endif



void setpumpstates(void)
{
	if(gettemp(solartemp)-(vars[30]) > gettemp(tankbottomtemp)){
	solarpumpstate = 1;
	dout_port |= 1<<solarpumpout;
	}
	
	if(gettemp(solartemp)-(vars[31]) < gettemp(tankbottomtemp)){
	solarpumpstate = 0;
	dout_port &= ~(1<<solarpumpout);
	}	
	
	if((gettemp(tankbottomtemp) < (vars[13]) && getshuntpos() > (vars[10])) && gettemp(insidetemp) < (vars[15])){
	heatpumpstate = 1;
	dout_port |= 1<<heatpumpout;
	}

	if(getshuntpos() < (vars[12]) && solarpumpstate){
	heatpumpstate = 0;
	dout_port &= ~(1<<heatpumpout);
	}

	if(getshuntpos() < (vars[11]) && !solarpumpstate){
	heatpumpstate = 0;
	dout_port &= ~(1<<heatpumpout);
	}

	if(gettemp(tankbottomtemp) > (vars[14])){
	heatpumpstate = 0;
	dout_port &= ~(1<<heatpumpout);
	}

	if((getshuntpos() > (vars[20])) && gettemp(insidetemp) < (vars[22])){
	electricstate = 1;
	dout_port |= 1<<electricout;
	}

	if(getshuntpos() < (vars[21]) || gettemp(insidetemp) > (vars[22]+vars[23])){
	electricstate = 0;
	dout_port &= ~(1<<electricout);
	}

	if(gettemp(outsidetemp) < (vars[29])){
	mainpumpstate = 1;
	dout_port |= 1<<mainpumpout;
	}

	if(gettemp(outsidetemp) > (vars[28])){
	mainpumpstate = 0;
	dout_port &= ~(1<<mainpumpout);
	}

#ifdef internal
	
	signed char fwt = 0;
	signed char fwsp = 0;
	
	fwsp = getforwardsp();
	
	fwt= gettempint(forwardtemp);

	if(fwt > fwsp+(vars[84]/2)){
	
		if((!shunttimer) && (shunttime > vars[80])){
			sdir = close;
			shunttimer = vars[80]+1;
		}	
	}

	if(fwt < fwsp-(vars[84]/2)){

		if((!shunttimer) && (shunttime < vars[81]-vars[80])){
			sdir = open;
			shunttimer = vars[80]+1;
		}	
	
	}
#endif

}



void LCD_drawschema(void)
{
FB_drawline(27+26,10,27+26,55,0); /* Tank */
FB_drawline(28+26,56,33+26,61,0);
FB_drawline(28+26,9,33+26,4,0);
FB_drawline(34+26,61,44+26,61,0);
FB_drawline(34+26,4,44+26,4,0);
FB_drawline(45+26,60,50+26,55,0);
FB_drawline(45+26,4,50+26,9,0);
FB_drawline(51+26,10,51+26,54,0);

FB_drawline(51+26,50,64+26,50,0); /* Pipes for solar */
FB_drawline(65+26,49,65+26,20,0);
FB_drawline(66+26,20,70+26,20,0);
FB_drawline(51+26,30,57+26,30,0);
FB_drawline(58+26,30,58+26,18,0);
FB_drawline(58+26,11,58+26,6,0);
FB_drawline(59+26,6,66+26,6,0);
FB_drawcircle(58+26,15,3);


FB_drawcircle(90+26,5,5);	/* Sun */
FB_drawline(80+26,5,83+26,5,0);
FB_drawline(82+26,0,84+26,1,0);
FB_drawline(82+26,11,84+26,9,0);

FB_drawline(67+26,5,70+26,3,0); /* Solar panel */
FB_drawline(67+26,6,70+26,20,0);
FB_drawline(71+26,4,74+26,18,0);
FB_drawline(71+26,21,74+26,19,0);

FB_drawline(65+26,50,75+26,50,0); /* Pipes for heat pump */
FB_drawline(96+26,50,100+26,50,0);
FB_drawline(100+26,49,100+26,30,0);
FB_drawline(99+26,30,67+26,30,0);
FB_drawline(63+26,30,59+26,30,0);

FB_drawrect(75+26,45,95+26,55,0,0); /* Heat pump */
FB_drawcircle(80+26,50,3);

FB_drawcircle(10+26,26,3);	/* Main pump */

FB_drawline(24+26,6,30+26,6,0);	/* Pipes for main pump */
FB_drawline(22+26,26,24+26,26,0);
FB_drawline(24+26,25,24+26,7,0);
FB_drawline(13+26,26,15+26,26,0);
FB_drawline(0,26,6+26,26,0);

FB_drawline(0,40,19+26,40,0);	/* Return water pipes */
FB_drawline(19+26,39,19+26,28,0);
FB_drawline(20+26,35,27+26,35,0);


FB_drawsym3wvalvedn(16+26, 24, 0);	/* shunt */



} 

void showpumps(void)
{
	if(solarpumpstate){
		FB_drawsympumpondn(57+26,15,0);
	}
	
	if(!(solarpumpstate)){
		FB_drawsympumpondn(57+26,15,1);
	}	
	
	if(heatpumpstate){
		FB_drawsymcomprt(78+26,48,0);
	}
	
	if(!(heatpumpstate)){
		FB_drawsymcomprt(78+26,48,1);
	}

	if(electricstate){
		FB_drawsymflash(88+26,47,0);

	}

	if(!(electricstate)){
		FB_drawsymflash(88+26,47,1);
	}

	if(mainpumpstate){
		FB_drawsympumponlf(9+26,25,0);
	}

	if(!(mainpumpstate)){
		FB_drawsympumponlf(9+26,25,1);
	}

}

void showtemps(void)
{
	char str[5];
	
	sprintf(str, "%2.1f", gettemp(tanktoptemp));
	FB_writestring(29+26,10,str);
	
	sprintf(str, "%2.1f", gettemp(tankmiddletemp));
	FB_writestring(29+26,27,str);
	
	sprintf(str, "%2.1f", gettemp(tankbottomtemp));
	FB_writestring(29+26,45,str);
	
	sprintf(str, "%2.1f", gettemp(solartemp));
	FB_writestring(79+26,14,str);

	sprintf(str, "Out:%2.1f", gettemp(outsidetemp));
	FB_writestring(4,56,str);

	sprintf(str, "In: %2.1f", gettemp(insidetemp));
	FB_writestring(4,0,str);

	sprintf(str, "%2.1f", gettemp(returntemp));
	FB_writestring(4,41,str);

	sprintf(str, "%3.0d%%", getshuntpos());
	FB_writestring(26,14,str);

#ifdef internal
	sprintf(str, "%2.1f", gettemp(forwardtemp));
	FB_writestring(4,17,str);
#endif

	sprintf(str, "(%2.1f)", getforwardsp());
	FB_writestring(0,27,str);
	
}


ISR(TIMER1_COMPA_vect)
{
	TCNT1 = 0;
	
	if(mcaltimer){
		mcaltimer--;
		dout_port |= 1<<scloseout;	/* Start shunt motor calibration */
	}
	
	if(!mcaltimer){
		if(shunttimer){
			shunttimer--;
			if(sdir == open){
				dout_port |= 1<<sopenout;
				shunttime++;
			}
		
			if(sdir == close){
				dout_port |= 1<<scloseout;
				shunttime--;
			}

		}

		if(!shunttimer){
			dout_port &= ~(1<<sopenout);
			dout_port &= ~(1<<scloseout);
	
		}
	}	

	if(!regtimer){
		setpumpstates();	
		regtimer = vars[83];
	
	}

	regtimer--;

}


int main(void)
{
dispmode = normal;

setup();

//LCD_update();

//menurender();
//LCD_drawschema();

	while(1){
		
		for(unsigned long i = 0; i < 900000; i++){

			//showtemps();
			//menurender();
			//LCD_update();
		
			if(!down_pin){
				keymanager(down);
				menurender();
				LCD_update();
				while(!down_pin);
			}
			if(!up_pin){
				keymanager(up);
				menurender();
				LCD_update();
				while(!up_pin);
			}
			if(!enter_pin){
				keymanager(enter);
				menurender();
				LCD_update();
				while(!enter_pin);
			}
			if(!escape_pin){
				keymanager(escape);
				if((dispmode == menu) || (dispmode == editvar) || dispmode == editcurve) menurender();
				LCD_update();
				while(!escape_pin);

			}

			if(dispmode == menu){
				menurender();
				LCD_update();
			}

		
		
		}	
	
		if(dispmode == normal){
			//LCD_init();

			FB_clear();
			showpumps();
			showtemps();
			LCD_drawschema();
			LCD_update();
		}
	}

	

}
